@@ -213,18 +213,12 @@ fn test_num_live_threads() {
     let pool = ThreadPool::new("pool1", 3).unwrap();
     sleep(100);
     assert_eq!(3, pool.num_live_threads());
-    pool.schedule(move || {
-        sleep(100);
-        panic!("ignore this panic");
-    });
-    pool.schedule(move || {
-        sleep(100);
-        panic!("ignore this panic");
-    });
-    pool.schedule(move || {
-        sleep(100);
-        panic!("ignore this panic");
-    });
+    for _ in 0..3 {
+        pool.schedule(move || {
+            sleep(100);
+            panic!("ignore this panic");
+        });
+    }
     sleep(200);
     assert_eq!(0, pool.num_live_threads());
     pool.schedule(move || {});
@@ -247,24 +241,29 @@ fn schedule_should_run_the_fn() {
 fn schedule_should_start_a_thread_if_none() {
     let pool = ThreadPool::new("pool1", 3).unwrap();
     sleep(100);
-    pool.schedule(move || {
-        sleep(100);
-        panic!("ignore this panic");
-    });
-    pool.schedule(move || {
-        sleep(100);
-        panic!("ignore this panic");
-    });
-    pool.schedule(move || {
-        sleep(100);
-        panic!("ignore this panic");
-    });
+    for _ in 0..3 {
+        pool.schedule(move || {
+            sleep(100);
+            panic!("ignore this panic");
+        });
+    }
     sleep(200);
     assert_eq!(0, pool.num_live_threads());
     pool.schedule(|| {});
     assert_eq!(3, pool.num_live_threads());
 }
 
+#[test]
+fn schedule_retries_when_queue_full() {
+    let pool = ThreadPool::new("test", 1).unwrap();
+    let before = Instant::now();
+    pool.schedule(move || sleep(100));
+    pool.schedule(move || sleep(100));
+    while pool.try_schedule(|| {}).is_ok() {}
+    pool.schedule(|| {});
+    assert_elapsed(before, 100..200);
+}
+
 #[test]
 fn try_schedule_should_run_the_fn() {
     let pool = ThreadPool::new("pool1", 1).unwrap();
@@ -282,14 +281,13 @@ fn try_schedule_should_run_the_fn() {
 fn try_schedule_queue_full() {
     let pool = ThreadPool::new("pool1", 1).unwrap();
     let before = Instant::now();
-    loop {
-        assert!(Instant::now() - before < Duration::from_millis(500));
-        let result = pool.try_schedule(move || sleep(500));
-        if result == Err(TryScheduleError::QueueFull) {
-            return;
-        }
-        result.unwrap();
-    }
+    pool.try_schedule(move || sleep(100)).unwrap();
+    while pool.try_schedule(|| {}).is_ok() {}
+    assert!(matches!(
+        pool.try_schedule(|| {}),
+        Err(TryScheduleError::QueueFull),
+    ));
+    assert_elapsed(before, 0..100);
 }
 
 #[test]
